@@ -4,7 +4,15 @@ use strict;
 use base qw/Class::Data::Inheritable Class::Accessor::Fast/;
 use NEXT;
 
-__PACKAGE__->mk_classdata('_config');
+__PACKAGE__->mk_classdata($_) for qw/_cache _config/;
+__PACKAGE__->_cache( [] );
+
+# note - see attributes(3pm)
+sub MODIFY_CODE_ATTRIBUTES {
+    my ( $class, $code, @attrs ) = @_;
+    push @{ $class->_cache }, [ $code, [@attrs] ];
+    return ();
+}
 
 =head1 NAME
 
@@ -33,7 +41,6 @@ Catalyst::Base - Catalyst Universal Base Class
 
     # Methods can be a request step
     $c->forward(qw/MyApp::Model::Something forward_to_me/);
-    MyApp->action( 'index.html' => \&MyApp::Model::Something::forward_to_me );
 
     # Or just methods
     print $c->comp('MyApp::Model::Something')->test;
@@ -88,7 +95,9 @@ sub config {
 
 =cut
 
-sub process { 1 }
+sub process { die ((ref $_[0] || $_[0])." did not override Catalyst::Base::process"); }
+
+=back
 
 =head1 SEE ALSO
 
@@ -97,6 +106,7 @@ L<Catalyst>.
 =head1 AUTHOR
 
 Sebastian Riedel, C<sri@cpan.org>
+Marcus Ramberg, C<mramberg@cpan.org>
 
 =head1 COPYRIGHT
 
